@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 import numpy as np
 from .utils import fft, ifft, ifftshift, FFT_GLOBAL
-from .utils import WARN, adm_cwt, adm_ssq, _process_fs_and_t
+from .utils import WARN, adm_cwt, adm_ssq, _process_fs_and_t, is_array_or_tensor
 from .utils import padsignal, process_scales, logscale_transition_idx
 from .utils import backend as S
 from .utils.backend import Q
@@ -12,15 +12,16 @@ from .wavelets import Wavelet
 def cwt(x, wavelet='gmw', scales='log-piecewise', fs=None, t=None, nv=32,
         l1_norm=True, derivative=False, padtype='reflect', rpadded=False,
         vectorized=True, astensor=True, cache_wavelet=None, order=0, average=None,
-        nan_checks=True, patience=0):
-    """Continuous Wavelet Transform, discretized, as described in
-    Sec. 4.3.3 of [1] and Sec. IIIA of [2]. Uses FFT convolution via frequency-
-    domain wavelets matching (padded) input's length.
+        nan_checks=None, patience=0):
+    """Continuous Wavelet Transform. Uses FFT convolution via frequency-domain
+    wavelets matching (padded) input's length.
+
+    Differentiability enabled via `os.environ['SSQ_GPU'] = '1'`.
 
     Uses `Wavelet.dtype` precision.
 
     # Arguments:
-        x: np.ndarray
+        x: np.ndarray / torch.Tensor
             Input vector(s), 1D / 2D.
 
             2D: does *not* do 2D CWT. Instead, treats dim0 as separate inputs,
@@ -107,12 +108,17 @@ def cwt(x, wavelet='gmw', scales='log-piecewise', fs=None, t=None, nv=32,
             > 0 computes `cwt` with higher-order GMWs. If tuple, computes
             `cwt` at each specified order. See `help(_cwt.cwt_higher_order)`.
 
+            NOTE: implementation may be not entirely correct. Specifically,
+            alignment by center frequency rather than scales may be optimal.
+
         average: bool / None
             Only used for tuple `order`; see `help(_cwt.cwt_higher_order)`.
 
-        nan_checks: bool (default True)
+        nan_checks: bool / None
             Checks whether input has `nan` or `inf` values, and zeros them.
-            `False` saves compute.
+            `False` saves compute. Doesn't support torch inputs.
+
+            Defaults to `True` for NumPy inputs, else `False`.
 
         patience: int / tuple[int, int]
             pyFFTW parameter for faster FFT on CPU; see `help(ssqueezepy.FFT)`.
@@ -127,24 +133,34 @@ def cwt(x, wavelet='gmw', scales='log-piecewise', fs=None, t=None, nv=32,
             differentiation (effectively, derivative of trigonometric
             interpolation; see [4]). Implements as described in Sec IIIB of [2].
 
-    # References:
-        1. Wavelet Tour of Signal Processing, 3rd ed. S. Mallat.
-        https://www.di.ens.fr/~mallat/papiers/WaveletTourChap1-2-3.pdf
+    # Note:
+        CWT is cross-correlation of wavelets with input. For zero-phase wavelets
+        (real-valued in Fourier), this is equivalent to convolution. All
+        ssqueezepy wavelets are zero-phase. If a custom general wavelet is
+        used, it must be conjugated in frequency, and it should *not* be used
+        with synchrosqueezing (see one-integral inverse References in `icwt`).
 
-        2. The Synchrosqueezing algorithm for time-varying spectral analysis:
+    # References:
+        1. The Synchrosqueezing algorithm for time-varying spectral analysis:
         robustness properties and new paleoclimate applications. G. Thakur,
         E. Brevdo, N.-S. Fučkar, and H.-T. Wu.
         https://arxiv.org/abs/1105.0010
 
-        3. Rectification of the Bias in the Wavelet Power Spectrum.
-        Y. Liu, X. S. Liang, R. H. Weisberg.
-        http://ocg6.marine.usf.edu/~liu/Papers/Liu_etal_2007_JAOT_wavelet.pdf
+        2. How to validate a wavelet filterbank (CWT)? John Muradeli.
+        https://dsp.stackexchange.com/a/86069/50076
+
+        3. Wavelet "center frequency" explanation? Relation to CWT scales?
+        John Muradeli.
+        https://dsp.stackexchange.com/a/76371/50076
 
         4. The Exponential Accuracy of Fourier and Chebyshev Differencing Methods.
         E. Tadmor.
         http://webhome.auburn.edu/~jzl0097/teaching/math_8970/Tadmor_86.pdf
 
-        5. Synchrosqueezing Toolbox, (C) 2014--present. E. Brevdo, G. Thakur.
+        5. Wavelet Tour of Signal Processing, 3rd ed. S. Mallat.
+        https://www.di.ens.fr/~mallat/papiers/WaveletTourChap1-2-3.pdf
+
+        6. Synchrosqueezing Toolbox, (C) 2014--present. E. Brevdo, G. Thakur.
         https://github.com/ebrevdo/synchrosqueezing/blob/master/synchrosqueezing/
         cwt_fw.m
     """
@@ -182,16 +198,21 @@ def cwt(x, wavelet='gmw', scales='log-piecewise', fs=None, t=None, nv=32,
                 dWx[idx] = ifft(dpsih * xh, axis=-1, astensor=True)
         return (Wx, dWx) if derivative else (Wx, None)
 
-    def _process_args(x, scales, nv, fs, t, wavelet, cache_wavelet):
-        if not isinstance(x, np.ndarray):
-            raise TypeError("`x` must be a numpy array (got %s)" % type(x))
+    def _process_args(x, scales, nv, fs, t, nan_checks, wavelet, cache_wavelet):
+        if not hasattr(x, 'ndim'):
+            raise TypeError("`x` must be a numpy array or torch Tensor "
+                            "(got %s)" % type(x))
         elif x.ndim not in (1, 2):
             raise ValueError("`x` must be 1D or 2D (got x.ndim == %s)" % x.ndim)
 
-        if nan_checks and (
-                np.isnan(x.max()) or np.isinf(x.max()) or np.isinf(x.min())):
-            WARN("found NaN or inf values in `x`; will zero")
-            replace_at_inf_or_nan(x, replacement=0.)
+        if nan_checks is None:
+            nan_checks = bool(isinstance(x, np.ndarray))
+        if nan_checks:
+            if not isinstance(x, np.ndarray):
+                raise ValueError("`nan_checks=True` requires NumPy input.")
+            elif np.isnan(x.max()) or np.isinf(x.max()) or np.isinf(x.min()):
+                WARN("found NaN or inf values in `x`; will zero")
+                replace_at_inf_or_nan(x, replacement=0.)
 
         if cache_wavelet:
             if isinstance(wavelet, (str, tuple)):
@@ -206,7 +227,7 @@ def cwt(x, wavelet='gmw', scales='log-piecewise', fs=None, t=None, nv=32,
         elif cache_wavelet is None:
             cache_wavelet = (not isinstance(wavelet, (str, tuple)) and vectorized)
 
-        if isinstance(scales, np.ndarray):
+        if not isinstance(scales, str):
             nv = None
 
         N = x.shape[-1]
@@ -223,23 +244,28 @@ def cwt(x, wavelet='gmw', scales='log-piecewise', fs=None, t=None, nv=32,
                                 astensor=astensor, **kw)
 
     (N, nv, dt, is_2D, cache_wavelet
-     ) = _process_args(x, scales, nv, fs, t, wavelet, cache_wavelet)
+     ) = _process_args(x, scales, nv, fs, t, nan_checks, wavelet, cache_wavelet)
 
     # process `wavelet`, get its `dtype`
     wavelet = _process_gmw_wavelet(wavelet, l1_norm)
     wavelet = Wavelet._init_if_not_isinstance(wavelet)
     dtype = wavelet.dtype
 
-    x = x.astype(dtype)
+    # cast to torch early if possible (keeps as NumPy if SSQ_GPU=0)
+    torch_supports_padding = bool(padtype in ('zero', 'reflect', None))
+    if torch_supports_padding:
+        x = S.asarray(x, dtype)
+    x = S.astype(x, dtype)
+
+    # pad, ensure correct data type
     if padtype is not None:
         xp, _, n1, _ = padsignal(x, padtype, get_params=True)
     else:
         xp = x
+    if not torch_supports_padding:
+        xp = S.asarray(xp, dtype)
 
-    # zero-mean `xp`, take to freq-domain
-    xp = S.asarray(xp)
-    x_mean = xp.mean(axis=-1)
-    xp -= (x_mean[:, None] if is_2D else x_mean)
+    # take to freq-domain
     xh = fft(xp, axis=-1, astensor=True)
     if is_2D:
         xh = xh[:, None]  # insert dim1 to broadcast wavelet `scales` along
@@ -295,7 +321,7 @@ def cwt(x, wavelet='gmw', scales='log-piecewise', fs=None, t=None, nv=32,
 
 
 def icwt(Wx, wavelet='gmw', scales='log-piecewise', nv=None, one_int=True,
-         x_len=None, x_mean=0, padtype='zero', rpadded=False, l1_norm=True):
+         x_len=None, x_mean=0, padtype='reflect', rpadded=False, l1_norm=True):
     """The inverse Continuous Wavelet Transform of `Wx`, via double or
     single integral.
 
@@ -303,6 +329,10 @@ def icwt(Wx, wavelet='gmw', scales='log-piecewise', nv=None, one_int=True,
         Wx: np.ndarray
             CWT computed via `ssqueezepy.cwt`.
 
+              - 2D: (n_scales, n_times)
+              - 3D: (n_inputs, n_scales, n_times).
+                Doesn't support `one_int=False`.
+
         wavelet: str / tuple[str, dict] / `wavelets.Wavelet`
             Wavelet sampled in Fourier frequency domain.
                 - str: name of builtin wavelet. `ssqueezepy.wavs()`
@@ -320,14 +350,16 @@ def icwt(Wx, wavelet='gmw', scales='log-piecewise', nv=None, one_int=True,
         one_int: bool (default True)
             Whether to use one-integral iCWT or double.
             Current one-integral implementation performs best.
-                - True: Eq 2.6, modified, of [3]. Explained in [4].
-                - False: Eq 4.67 of [1]. Explained in [5].
+                - True: Eq 2.6, modified, of [6]. Explained in [1].
+                - False: Eq 4.67 of [3]. Explained in [2].
 
         x_len: int / None. Length of `x` used in forward CWT, if different
             from Wx.shape[1] (default if None).
 
         x_mean: float. mean of original `x` (not picked up in CWT since it's an
             infinite scale component). Default 0.
+            Note: if `Wx` is 3D, `x_mean` should be 1D (`x.mean()` along samples
+            axis).
 
         padtype: str
             Pad scheme to apply on input, in case of `one_int=False`.
@@ -341,35 +373,40 @@ def icwt(Wx, wavelet='gmw', scales='log-piecewise', nv=None, one_int=True,
 
     # Returns:
         x: np.ndarray
-            The signal, as reconstructed from Wx.
+            The signal(s), as reconstructed from Wx.
+
+            If `Wx` is 3D, `x` has shape `(n_inputs, n_times)`.
 
     # References:
-        1. Wavelet Tour of Signal Processing, 3rd ed. S. Mallat.
+        1. One integral inverse CWT. John Muradeli.
+        https://dsp.stackexchange.com/a/76239/50076
+
+        2. Inverse CWT derivation. John Muradeli.
+        https://dsp.stackexchange.com/a/71148/50076
+
+        3. Wavelet Tour of Signal Processing, 3rd ed. S. Mallat.
         https://www.di.ens.fr/~mallat/papiers/WaveletTourChap1-2-3.pdf
 
-        2. The Synchrosqueezing algorithm for time-varying spectral analysis:
+        4. Why iCWT may be inexact. John Muradeli.
+        https://dsp.stackexchange.com/a/87104/50076
+
+        5. The Synchrosqueezing algorithm for time-varying spectral analysis:
         robustness properties and new paleoclimate applications. G. Thakur,
         E. Brevdo, N.-S. Fučkar, and H.-T. Wu.
         https://arxiv.org/abs/1105.0010
 
-        3. Synchrosqueezed Wavelet Transforms: a Tool for Empirical Mode
+        6. Synchrosqueezed Wavelet Transforms: a Tool for Empirical Mode
         Decomposition. I. Daubechies, J. Lu, H.T. Wu.
         https://arxiv.org/pdf/0912.2437.pdf
 
-        4. One integral inverse CWT. OverLordGoldDragon.
-        https://dsp.stackexchange.com/a/71274/50076
-
-        5. Inverse CWT derivation. OverLordGoldDragon.
-        https://dsp.stackexchange.com/a/71148/50076
-
-        6. Synchrosqueezing Toolbox, (C) 2014--present. E. Brevdo, G. Thakur.
+        7. Synchrosqueezing Toolbox, (C) 2014--present. E. Brevdo, G. Thakur.
         https://github.com/ebrevdo/synchrosqueezing/blob/master/synchrosqueezing/
-        synsq_cwt_fw.m
+        synsq_cwt_iw.m
     """
     #### Prepare for inversion ###############################################
-    na, n = Wx.shape
+    *_, na, n = Wx.shape
     x_len = x_len or n
-    if not isinstance(scales, np.ndarray) and nv is None:
+    if not is_array_or_tensor(scales) and nv is None:
         nv = 32  # must match forward's; default to `cwt`'s
 
     wavelet = _process_gmw_wavelet(wavelet, l1_norm)
@@ -387,8 +424,8 @@ def icwt(Wx, wavelet='gmw', scales='log-piecewise', nv=None, one_int=True,
                   padtype=padtype, rpadded=rpadded, l1_norm=l1_norm)
 
         idx = logscale_transition_idx(scales)
-        x  = icwt(Wx[:idx], scales=scales[:idx], **kw)
-        x += icwt(Wx[idx:], scales=scales[idx:], **kw)
+        x  = icwt(Wx[..., :idx, :], scales=scales[:idx], **kw)
+        x += icwt(Wx[..., idx:, :], scales=scales[idx:], **kw)
         return x
     ##########################################################################
 
@@ -396,6 +433,8 @@ def icwt(Wx, wavelet='gmw', scales='log-piecewise', nv=None, one_int=True,
     if one_int:
         x = _icwt_1int(Wx, scales, scaletype, l1_norm)
     else:
+        if Wx.ndim == 3:
+            raise NotImplementedError("batched `Wx` requires `one_int=True`.")
         x = _icwt_2int(Wx, scales, scaletype, l1_norm,
                        wavelet, x_len, padtype, rpadded)
 
@@ -403,11 +442,12 @@ def icwt(Wx, wavelet='gmw', scales='log-piecewise', nv=None, one_int=True,
     Cpsi = (adm_ssq(wavelet) if one_int else
             adm_cwt(wavelet))
     if scaletype == 'log':
-        # Eq 4.67 in [1]; Theorem 4.5 in [1]; below Eq 14 in [2]
+        # Eq 4.67 in [3]; Theorem 4.5 in [3]; below Eq 14 in [5]
         # ln(2**(1/nv)) == ln(2)/nv == diff(ln(scales))[0]
         x *= (2 / Cpsi) * np.log(2 ** (1 / nv))
     else:
-        x *= (2 / Cpsi)
+        # unclear why the `pi/4` here but it improves inversion
+        x *= (2 / Cpsi) * np.pi / 4
 
     x += x_mean  # CWT doesn't capture mean (infinite scale)
     return x
@@ -439,7 +479,7 @@ def _icwt_2int(Wx, scales, scaletype, l1_norm, wavelet, x_len,
 def _icwt_1int(Wx, scales, scaletype, l1_norm):
     """One-integral iCWT; assumes analytic wavelet."""
     norm = _icwt_norm(scaletype, l1_norm)
-    return (Wx.real / norm(scales)).sum(axis=0)
+    return (Wx.real / norm(scales)).sum(axis=-2)
 
 
 def _icwt_norm(scaletype, l1_norm):
