@@ -738,7 +738,7 @@ def _moments_to_cumulants(moments):
 
     for n in range(1, len(moments)):
         coeff = 0
-        for k in range(1, n - 1):
+        for k in range(1, n):
             coeff += nCk(n - 1, k - 1
                          ) * cumulants[k] * (moments[n - k] / moments[0])
         cumulants[n] = (moments[n] / moments[0]) - coeff
