@@ -274,17 +274,17 @@ def infer_scaletype(scales):
         raise TypeError("`scales.dtype` must be np.float32 or np.float64 "
                         "(got %s)" % scales.dtype)
 
-    th_log = 1e-15 if scales.dtype == np.float64 else 4e-7
+    th_log = 4e-15 if scales.dtype == np.float64 else 8e-7
     th_lin = th_log * 1e3  # less accurate for some reason
 
-    if np.mean(np.abs(np.diff(scales, 2, axis=0))) < th_lin:
-        scaletype = 'linear'
-        nv = None
-
-    elif np.mean(np.abs(np.diff(np.log(scales), 2, axis=0))) < th_log:
+    if np.mean(np.abs(np.diff(np.log(scales), 2, axis=0))) < th_log:
         scaletype = 'log'
         # ceil to avoid faulty float-int roundoffs
-        nv = int(np.round(1 / np.diff(np.log2(scales), axis=0)[0]))
+        nv = int(np.round(1 / np.diff(np.log2(scales), axis=0)[0].squeeze()))
+
+    elif np.mean(np.abs(np.diff(scales, 2, axis=0))) < th_lin:
+        scaletype = 'linear'
+        nv = None
 
     elif logscale_transition_idx(scales) is None:
         raise ValueError("could not infer `scaletype` from `scales`; "
@@ -588,7 +588,7 @@ def integrate_analytic(int_fn, nowarn=False):
     Integrates near zero separately in log space (useful for e.g. 1/x).
     """
     def _est_arr(mxlim, N):
-        t = np.linspace(mxlim, .1, N, endpoint=False)[::-1]
+        t = np.linspace(mxlim, .1, N, endpoint=False)[::-1].copy()
         arr = int_fn(t)
 
         max_idx = np.argmax(arr)
@@ -620,11 +620,11 @@ def integrate_analytic(int_fn, nowarn=False):
         # (.001 to .1 may not be negligible, however; better captured by logspace)
         t = np.logspace(-15, -1, 1000)
         arr = int_fn(t)
-        return integrate.trapz(arr, t)
+        return integrate.trapezoid(arr, t)
 
     int_nz = _integrate_near_zero()
     arr, t = _find_convergent_array()
-    return integrate.trapz(arr, t) + int_nz
+    return integrate.trapezoid(arr, t) + int_nz
 
 
 def find_max_scale_alt(wavelet, N, min_cutoff=.1, max_cutoff=.8):
