@@ -139,7 +139,7 @@ def _buffer_gpu(x, seg_len, n_segs, hop_len, s20, s21, modulated=False, out=None
 
 
 def unbuffer(xbuf, window, hop_len, n_fft, N, win_exp=1):
-    """Undoes `buffer`, per padding logic used in `stft`:
+    """Undoes `buffer` (minus unpadding), per padding logic used in `stft`:
         (N, n_fft) : logic
          even, even: left = right + 1
              (N, n_fft, len(xp), pl, pr) -> (128, 120, 247, 60, 59)
@@ -160,10 +160,9 @@ def unbuffer(xbuf, window, hop_len, n_fft, N, win_exp=1):
         window = 1
     elif win_exp != 1:
         window = window ** win_exp
-    x = np.zeros(N + n_fft - 1)
+    x = np.zeros(N + n_fft - 1, dtype=xbuf.dtype)
 
     _overlap_add(x, xbuf, window, hop_len, n_fft)
-    x = x[n_fft//2 : -((n_fft - 1)//2)]
     return x
 
 
@@ -172,7 +171,7 @@ def window_norm(window, hop_len, n_fft, N, win_exp=1):
     wn = np.zeros(N + n_fft - 1)
 
     _window_norm(wn, window, hop_len, n_fft, win_exp)
-    return wn[n_fft//2 : -((n_fft - 1)//2)]
+    return wn
 
 
 @jit(nopython=True, cache=True)
@@ -209,8 +208,8 @@ def window_resolution(window):
     apsi2   = np.abs(window)**2
     apsih2s = np.abs(psihs)**2
 
-    var_w = integrate.trapz(ws**2 * apsih2s, ws) / integrate.trapz(apsih2s, ws)
-    var_t = integrate.trapz(t**2  * apsi2, t)    / integrate.trapz(apsi2, t)
+    var_w = integrate.trapezoid(ws**2 * apsih2s, ws) / integrate.trapezoid(apsih2s, ws)
+    var_t = integrate.trapezoid(t**2  * apsi2, t)    / integrate.trapezoid(apsi2, t)
 
     std_w, std_t = np.sqrt(var_w), np.sqrt(var_t)
     harea = std_w * std_t
@@ -227,11 +226,11 @@ def window_area(window, time=True, frequency=False):
 
     if time:
         t = np.arange(-len(window)/2, len(window)/2, step=1)
-        at = integrate.trapz(np.abs(window)**2, t)
+        at = integrate.trapezoid(np.abs(window)**2, t)
     if frequency:
         ws = fftshift(_xifn(1, len(window)))
         apsih2s = np.abs(fftshift(fft(window)))**2
-        aw = integrate.trapz(apsih2s, ws)
+        aw = integrate.trapezoid(apsih2s, ws)
 
     if time and frequency:
         return at, aw
