@@ -597,7 +597,7 @@ def viz_cwt_higher_order(Wx_k, scales=None, wavelet=None, **imshow_kw):
             title = "abs(CWT), order={}{}".format(k, title_append)
             imshow(Wx, abs=1, title=title, **imshow_kw)
 
-        Wx_ka = np.mean(np.vstack([Wx_k]), axis=0)
+        Wx_ka = np.mean(np.abs(np.vstack([Wx_k])), axis=0)
         order_str = ','.join(map(str, range(len(Wx_k))))
         title = "abs(CWT), orders {} avg{}".format(order_str, title_append)
         imshow(Wx_ka, abs=1, title=title, **imshow_kw)
@@ -634,13 +634,15 @@ def viz_gmw_orders(N=1024, n_orders=3, scale=5, gamma=3, beta=60,
 #### Visual tools ## messy code ##############################################
 def imshow(data, title=None, show=1, cmap=None, norm=None, complex=None, abs=0,
            w=None, h=None, ridge=0, ticks=1, borders=1, aspect='auto', ax=None,
-           fig=None, yticks=None, xticks=None, xlabel=None, ylabel=None, **kw):
+           fig=None, yticks=None, xticks=None, xlabel=None, ylabel=None,
+           norm_scaling=1, **kw):
     """
     norm: color norm, tuple of (vmin, vmax)
     abs: take abs(data) before plotting
     ticks: False to not plot x & y ticks
     borders: False to not display plot borders
     w, h: rescale width & height
+    norm_scaling: multiplies `norm`, even if `norm` is None (multiplies default)
     kw: passed to `plt.imshow()`
 
     others
@@ -661,10 +663,12 @@ def imshow(data, title=None, show=1, cmap=None, norm=None, complex=None, abs=0,
                       (0, mx))
     else:
         vmin, vmax = norm
+    vmin *= norm_scaling
+    vmax *= norm_scaling
 
     # colormap
     import matplotlib as mpl
-    mpl33 = bool(float(mpl.__version__[:3]) >= 3.3)
+    mpl33 = tuple(map(int, mpl.__version__.split('.')[:2])) >= (3, 3)
     if cmap is None:
         cmap = (('turbo' if mpl33 else 'jet') if abs else
                 'bwr')
@@ -719,7 +723,7 @@ def plot(x, y=None, title=None, show=0, ax_equal=False, complex=0, abs=0,
     """
     norm: color norm, tuple of (vmin, vmax)
     abs: take abs(data) before plotting
-    complex: plot `x.real` & `x.imag`
+    complex: plot `x.real` & `x.imag`; `2` to also plot `abs(x)`
     ticks: False to not plot x & y ticks
     w, h: rescale width & height
     kw: passed to `plt.imshow()`
@@ -750,6 +754,8 @@ def plot(x, y=None, title=None, show=0, ax_equal=False, complex=0, abs=0,
     if complex:
         ax.plot(x, y.real, color='tab:blue', **kw)
         ax.plot(x, y.imag, color='tab:orange', **kw)
+        if complex == 2:
+            ax.plot(x, np.abs(y), color='k', linestyle='--', **kw)
         if c_annot:
             _kw = dict(fontsize=15, xycoords='axes fraction', weight='bold')
             ax.annotate("real", xy=(.93, .95), color='tab:blue', **_kw)
@@ -761,10 +767,13 @@ def plot(x, y=None, title=None, show=0, ax_equal=False, complex=0, abs=0,
     if dx1:
         ax.set_xticks(np.arange(len(x)))
 
+    # styling
     if vlines:
-        vhlines(vlines, kind='v')
+        _vhlines(vlines, kind='v', ax=ax)
     if hlines:
-        vhlines(hlines, kind='h')
+        _vhlines(hlines, kind='h', ax=ax)
+    if abs and ylims is None:
+        ylims = (0, None)
 
     ticks = ticks if isinstance(ticks, (list, tuple)) else (ticks, ticks)
     if not ticks[0]:
@@ -875,9 +884,9 @@ def scat(x, y=None, title=None, show=0, ax_equal=False, s=18, w=None, h=None,
 
     _maybe_title(title, ax=ax)
     if vlines:
-        vhlines(vlines, kind='v')
+        _vhlines(vlines, kind='v', ax=ax)
     if hlines:
-        vhlines(hlines, kind='h')
+        _vhlines(hlines, kind='h', ax=ax)
     _scale_plot(fig, ax, show=show, ax_equal=ax_equal, w=w, h=h,
                 xlims=xlims, ylims=ylims, dx1=(len(x) if dx1 else 0),
                 xlabel=xlabel, ylabel=ylabel, auto_xlims=auto_xlims)
@@ -891,12 +900,24 @@ def plotscat(*args, **kw):
         plt.show()
 
 
-def hist(x, bins=500, title=None, show=0, stats=0):
+def hist(x, bins=500, title=None, show=0, stats=0, ax=None, fig=None,
+         w=1, h=1, xlims=None, ylims=None, xlabel=None, ylabel=None):
+    """Histogram. `stats=True` to print mean, std, min, max of `x`."""
+    def _fmt(*nums):
+        return [(("%.3e" % n) if (abs(n) > 1e3 or abs(n) < 1e-3) else
+                 ("%.3f" % n)) for n in nums]
+
+    ax  = ax  or plt.gca()
+    fig = fig or plt.gcf()
+
     x = np.asarray(x)
-    _ = plt.hist(x.ravel(), bins=bins)
-    _maybe_title(title)
+    _ = ax.hist(x.ravel(), bins=bins)
+    _maybe_title(title, ax=ax)
+    _scale_plot(fig, ax, show=show, w=w, h=h, xlims=xlims, ylims=ylims,
+                xlabel=xlabel, ylabel=ylabel)
     if show:
         plt.show()
+
     if stats:
         mu, std, mn, mx = (x.mean(), x.std(), x.min(), x.max())
         print("(mean, std, min, max) = ({}, {}, {}, {})".format(
@@ -904,8 +925,8 @@ def hist(x, bins=500, title=None, show=0, stats=0):
         return mu, std, mn, mx
 
 
-def vhlines(lines, kind='v'):
-    lfn = plt.axvline if kind=='v' else plt.axhline
+def _vhlines(lines, kind='v', ax=None):
+    lfn = getattr(plt if ax is None else ax, f'ax{kind}line')
 
     if not isinstance(lines, (list, tuple)):
         lines, lkw = [lines], {}
@@ -928,6 +949,8 @@ def _fmt(*nums):
 
 def _ticks(xticks, yticks, ax):
     def fmt(ticks):
+        if all(isinstance(h, str) for h in ticks):
+            return "%s"
         return ("%.d" if all(float(h).is_integer() for h in ticks) else
                 "%.2f")
 
